@@ -1,7 +1,5 @@
 from __future__ import annotations
-
 import abc
-import inspect
 import typing
 
 from sqlalchemy import orm
@@ -20,12 +18,18 @@ E = typing.TypeVar("E")
 
 
 class Resource(abc.ABC, typing.Generic[T]):
-    @classmethod
-    def resource_name(cls) -> str:
-        if inspect.isabstract(cls):
-            return cls.__name__
-        else:
-            return _get_next_descendant_of(cls, ancestor=Resource).__name__
+    @abc.abstractmethod
+    def close(self) -> None:
+        raise NotImplementedError
+
+    @property
+    @abc.abstractmethod
+    def name(self) -> str:
+        raise NotImplementedError
+
+    @abc.abstractmethod
+    def open(self) -> Resource[T]:
+        raise NotImplementedError
 
     @abc.abstractmethod
     def rollback(self) -> None:
@@ -38,7 +42,7 @@ class Resource(abc.ABC, typing.Generic[T]):
     def __eq__(self, other: object) -> bool:
         if other.__class__ is self.__class__:
             # noinspection PyTypeChecker
-            return self.resource_name == typing.cast(Resource[typing.Any], other).resource_name
+            return self.name == typing.cast(Resource, other).name
         else:
             return NotImplemented
 
@@ -50,13 +54,13 @@ class Resource(abc.ABC, typing.Generic[T]):
             return not result
 
     def __hash__(self) -> int:
-        return hash(self.resource_name)
+        return hash(self.name)
 
     def __repr__(self) -> str:
-        return f"{self.__class__.__name__}: {self.__class__.resource_name()}"
+        return f"{self.__class__.__name__}({self.name!r})"
 
 
-class Repository(Resource[E], abc.ABC, typing.Generic[E]):
+class Repository(abc.ABC, typing.Generic[E]):
     """Interface to access elements of a collection"""
 
     @abc.abstractmethod
@@ -80,48 +84,97 @@ class Repository(Resource[E], abc.ABC, typing.Generic[E]):
         raise NotImplementedError
 
 
-class SqlAlchemyRepository(Repository[E], abc.ABC, typing.Generic[E]):
-    def __init__(self, session: orm.Session):
+class SqlAlchemyRepository(Resource[E], Repository[E], typing.Generic[E]):
+    def __init__(
+        self,
+        name: str,
+        entity: typing.Type[E],
+        session: orm.Session,
+    ):
+        super().__init__()
+
+        self._entity = entity
+        self._name = name
         self._session = session
 
-    def add(self, item: E, /) -> E:
-        self.session.add(item)
-        return item
+        self.in_transaction = False
 
-    def all(self) -> typing.Generator[E, None, None]:
-        return self.session.query(self.entity_type).all()
+    def __enter__(self):
+        return self.open()
 
-    def delete(self, item: E, /) -> E:
-        self.session.delete(item)
-        return item
+    def __exit__(self, *args):
+        return self.close()
 
-    def get(self, item_id: typing.Any, /) -> E:
-        return self.session.query(self.entity_type).get(item_id)
+    # RESOURCE METHODS
+    def close(self) -> None:
+        self.in_transaction = False
+        # self._session.close()
 
     @property
-    @abc.abstractmethod
-    def entity_type(self) -> typing.Type[E]:
-        raise NotImplementedError
+    def name(self) -> str:
+        return self._name
+
+    def open(self) -> SqlAlchemyRepository[E]:
+        self.in_transaction = True
+        return self
 
     def rollback(self) -> None:
-        self.session.rollback()
-
-    @property
-    def session(self) -> orm.Session:
-        return self._session
+        if self.in_transaction:
+            self._session.rollback()
+        else:
+            raise exceptions.MissingTransactionBlock(
+                "Attempted to rollback a repository outside of a transaction."
+            )
 
     def save(self) -> None:
-        self.session.commit()
+        if self.in_transaction:
+            self._session.commit()
+        else:
+            raise exceptions.MissingTransactionBlock(
+                "Attempted to save a repository outside of a transaction."
+            )
+    # END RESOURCE METHODS
+
+    # REPOSITORY METHODS
+    def all(self) -> typing.Generator[E, None, None]:
+        return self._session.query(self._entity).all()
+
+    def add(self, item: E, /) -> E:
+        if self.in_transaction:
+            self._session.add(item)
+            return item
+        else:
+            raise exceptions.MissingTransactionBlock(
+                "Attempted to edit repository outside of a transaction."
+            )
+
+    def delete(self, item: E, /) -> E:
+        if self.in_transaction:
+            self._session.delete(item)
+            return item
+        else:
+            raise exceptions.MissingTransactionBlock(
+                "Attempted to edit repository outside of a transaction."
+            )
 
     def update(self, item: E, /) -> E:
-        self.session.merge(item)
-        return item
+        if self.in_transaction:
+            self._session.merge(item)
+            return item
+        else:
+            raise exceptions.MissingTransactionBlock(
+                "Attempted to edit repository outside of a transaction."
+            )
+
+    def get(self, item_id: typing.Any, /) -> E:
+        return self._session.query(self._entity).get(item_id)
+    # END REPOSITORY METHODS
 
     def where(self, predicate: typing.Any, /) -> typing.List[E]:
-        return self.session.query(self.entity_type).filter(predicate).all()
+        return self._session.query(self._entity).filter(predicate).all()
 
 
-class DictRepository(Repository[E], abc.ABC, typing.Generic[E]):
+class DictRepository(Resource[E], Repository[E], typing.Generic[E]):
     """Repository implementation based on a dictionary
 
     This exists primarily to make testing in client code simpler.  It was not designed for efficiency.
@@ -130,17 +183,37 @@ class DictRepository(Repository[E], abc.ABC, typing.Generic[E]):
     def __init__(
         self,
         *,
+        name: str,
         initial_values: typing.Dict[typing.Hashable, E],
         key_fn: typing.Callable[[E], typing.Hashable],
     ):
-        super().__init__()
-
+        self._name = name
         self._previous_state = initial_values
         self._current_state = initial_values.copy()
         self._key_fn = key_fn
 
         self.events: typing.List[typing.Tuple[str, typing.Dict[str, typing.Any]]] = []
 
+    def __enter__(self):
+        return self.open()
+
+    def __exit__(self, *args):
+        self.rollback()
+        return self.close()
+
+    # RESOURCE METHODS
+    def close(self) -> None:
+        self.events.append(("close", {}))
+        self._current_state = {}
+
+    @property
+    def name(self) -> str:
+        return self._name
+
+    def open(self) -> DictRepository[T]:
+        self.events = [("open", {})]
+        return self
+
     def rollback(self) -> None:
         self.events.append(("rollback", {}))
         self._current_state = self._previous_state.copy()
@@ -148,7 +221,9 @@ class DictRepository(Repository[E], abc.ABC, typing.Generic[E]):
     def save(self) -> None:
         self.events.append(("save", {}))
         self._previous_state = self._current_state.copy()
+    # END RESOURCE METHODS
 
+    # REPOSITORY METHODS
     def add(self, item: E, /) -> E:
         self.events.append(("add", {"item": item}))
         key = self._key_fn(item)
@@ -174,15 +249,4 @@ class DictRepository(Repository[E], abc.ABC, typing.Generic[E]):
     def get(self, item_id: typing.Any, /) -> E:
         self.events.append(("get", {"item_id": item_id}))
         return self._current_state[item_id]
-
-
-def _get_next_descendant_of(
-    cls: typing.Type[typing.Any], ancestor: typing.Type[typing.Any]
-) -> typing.Type[typing.Any]:
-    try:
-        return next(c for c in cls.__mro__[1:] if ancestor in c.__mro__)
-    except StopIteration:
-        raise exceptions.NoCommonAncestor(
-            f"Cannot find a common ancestor of {ancestor.__name__} for class {cls.__name__} in its "
-            f"MRO.  The {cls.__name__}'s MRO is as follows: {', '.join(c.__name__ for c in cls.__mro__)}."
-        )
+    # END REPOSITORY METHODS
